/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.JTextComponent;
import javax.swing.tree.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CollectionConfiguration;
import org.greenstone.gatherer.cdm.CollectionDesignManager;
import org.greenstone.gatherer.cdm.Control;
import org.greenstone.gatherer.gui.EmailField;
import org.greenstone.gatherer.msm.MSMUtils;
import org.greenstone.gatherer.util.Utility;
/** This class is responsible for generating the necessary GUI components. It does this by calling the getEditControls() method of the appropriate class (i.e. IndexManager for index related edits). It also is in charge of correctly adding (and removing) listeners, using phrases from the <strong>Dictionary</strong> rather than the text keys inserted in the component classes, and several other aspects of the design page, including the config file section tree.
* @author John Thompson, Greenstone Digital Library, University of Waikato
* @version 2.3d
*/
public class GeneralManager
    extends JPanel {
    /** The available subscreens. */
    static final private String CONTENTS[] = {"General", "Plugins", "SearchTypes", "Indexes", "Subcollections", "SuperCollection", "Classifiers", "Formats", "Translation",  "MetadataSets"};
    /** The preferred size of the collection design module screen real-estate. */
    static final private Dimension SIZE = new Dimension(760, 500);
    /** The preferred size of the contents tree. */
    static final private Dimension TREE_SIZE = new Dimension(200, 500);
    /** The preferred size of label. */
    static final private Dimension LABEL_SIZE = new Dimension(200,25);
    /** The controls used to modify the general options. */
    private Control controls;
    /** The panel apon which is rendered the currently selected section screen. */
    private Control view = null;
    /** A tree to serve as a 'table of contents' for this design tool. We decided on a tree rather than a list, as it allows us to break sections into subsections if they become to complicated. */
    private DesignTree tree;
    /** Constructor. */
    public GeneralManager() {
	super();
	Gatherer.println("GeneralManager: Main GUI components created.");
	// Assignments
	this.controls = new GeneralControl();

	// Creation
	JPanel tree_pane = new JPanel();
	JLabel title = new JLabel(get("Design_Topics"));
	tree = new DesignTree();
	view = controls;

	// Connect
	tree.addTreeSelectionListener(new TreeListener());

	// Layout
	tree_pane.setLayout(new BorderLayout());
	tree_pane.setPreferredSize(TREE_SIZE);
	tree_pane.add(title, BorderLayout.NORTH);
	tree_pane.add(new JScrollPane(tree), BorderLayout.CENTER);

	setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	setLayout(new BorderLayout());
	add(tree_pane, BorderLayout.WEST);
	add((JPanel)view, BorderLayout.CENTER);
    }

    /** Destructor. */
    public void destroy() {
	controls.destroy();
	controls = null;
	tree = null;
	view = null;
    }

    /** Force the display to show a certain pane of controls.
     * @param type a String giving the name of the information manager or view we wish to display
     */
    public void setSelectedView(String type) {
	tree.setSelectedView(type);
    }

    /** Refresh the values on our controls that could be stale due to changes in other components. */
    public void gainFocus() {
	if(view != null) {
	    view.gainFocus();
	}
    }

    /** Saves the state of the controls in the current view. */
    public void loseFocus() {
	if(view != null) {
	    view.loseFocus();
	}
    }

    /** Overloaded to call get with both a key and an empty argument array.
     * @param key a String which is mapped to a key within the dictionary
     * @return a String which has been referenced by the key and that either contains no argument fields, or has had the argument fields automatically populated with arguments provided in the get call
     */
    private String get(String key) {
	return get(key, null);
    }

    /** Used to retrieve a property value from the Locale specific ResourceBundle, based upon the key and arguments supplied. If the key cannot be found or if some other part of the call fails a default (English) error message is returned. <BR>
     * Here the get recieves a second argument which is an array of Strings used to populate argument fields, denoted {<I>n</I>}, within the value String returned. Note that argument numbers greater than or equal to 32 are automatically mapped to the formatting String named Farg<I>n</I>.
     * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
     * @param args A <strong>String[]</strong> used to populate argument fields within the complete String.
     * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.Dictionary
     */
    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "CDM.GUI." + key;
	}
	return Gatherer.dictionary.get(key, args);
    }

    /** This class is resposible for generating the controls for the editing of general options.
     * @return the Control for editing the general options
     */
    private Control getControls() {
	return controls;
    }

    /** This class represents the visual component of the general options stored in the CollectionDesignManager. */
    private class GeneralControl
	extends JPanel 
	implements Control {
	private boolean ready = false;
	private CollectionMeta beta_collectionmeta;
	private CollectionMeta collection_extra_collectionmeta;
	private CollectionMeta collection_name_collectionmeta;
	private CollectionMeta creator_collectionmeta;
	private CollectionMeta icon_collection_collectionmeta;
	private CollectionMeta icon_collection_small_collectionmeta;
	private CollectionMeta maintainer_collectionmeta;
	private CollectionMeta public_collectionmeta;
	/** The creators email. */
	private EmailField creator_emailfield;
	/** The maintainers email. */
	private EmailField maintainer_emailfield;
	/** The checkbox controlling the state of the collection. */
	private JCheckBox beta_checkbox;
	/** The checkbox controlling public access to the collection. */
	private JCheckBox public_checkbox;
	private JLabel creator_label;
	private JLabel description_label;
	private JLabel icon_label;
	private JLabel maintainer_label;
	private JLabel name_label;
	private JLabel small_icon_label;
	private JLabel title_label;
	/** The text field used to edit the file name of the collections icon. */
	private JTextField icon_textfield;
	/** The text field used to edit the collections title. */
	private JTextField name_textfield;
	/** The text field used to edit the file name of the collections small icon. */
	private JTextField small_icon_textfield;
	/** A text area used to modify the collection description. */
	private JTextArea description_textarea;
	private JTextArea instructions_textarea;
	/** Constructor. */
	public GeneralControl() {
	    super();
	    // Retrieve some of the model elements, those we know aren't language dependant
	    beta_collectionmeta = new CollectionMeta(CollectionDesignManager.collect_config.getBeta());
	    public_collectionmeta = new CollectionMeta(CollectionDesignManager.collect_config.getPublic());
	   
	    // Creation
	    JPanel instruction_panel = new JPanel();
	    title_label = new JLabel("CDM.General.Title");
	    title_label.setHorizontalAlignment(JLabel.CENTER);
	    instructions_textarea = new JTextArea("CDM.General.Instructions");
	    instructions_textarea.setCaretPosition(0);
	    instructions_textarea.setEditable(false);
	    instructions_textarea.setLineWrap(true);
	    instructions_textarea.setRows(6);
	    instructions_textarea.setWrapStyleWord(true);

	    JPanel all_details_panel = new JPanel();
	    JPanel details_panel = new JPanel();
	    JPanel creator_panel = new JPanel();
	    creator_label = new JLabel("CDM.General.Email.Creator");
	    creator_label.setPreferredSize(LABEL_SIZE);
	    creator_emailfield = new EmailField(Gatherer.config.getColor("coloring.error_background", false));
	    JPanel maintainer_panel = new JPanel();
	    maintainer_label = new JLabel("CDM.General.Email.Maintainer");
	    maintainer_label.setPreferredSize(LABEL_SIZE);
	    maintainer_emailfield = new EmailField(Gatherer.config.getColor("coloring.error_background", false));
	    public_checkbox = new JCheckBox("CDM.General.Access", public_collectionmeta.getValue(CollectionMeta.TEXT).equals(CollectionConfiguration.TRUE_STR));
	    beta_checkbox = new JCheckBox("CDM.General.Beta", beta_collectionmeta.getValue(CollectionMeta.TEXT).equals(CollectionConfiguration.TRUE_STR));
	    JPanel name_panel = new JPanel();
	    name_label = new JLabel("CDM.General.Collection_Name");
	    name_label.setPreferredSize(LABEL_SIZE);
	    name_textfield = new JTextField();
	    JPanel icon_panel = new JPanel();
	    icon_label = new JLabel("CDM.General.Icon_Collection");
	    icon_label.setPreferredSize(LABEL_SIZE);
	    icon_textfield = new JTextField();
	    JPanel small_icon_panel = new JPanel();
	    small_icon_label = new JLabel("CDM.General.Icon_Collection_Small");
	    small_icon_label.setPreferredSize(LABEL_SIZE);
	    small_icon_textfield = new JTextField();
	    JPanel description_panel = new JPanel();
	    description_label = new JLabel("CDM.General.Collection_Extra");
	    description_label.setPreferredSize(LABEL_SIZE);
	    description_textarea = new JTextArea();
	    description_textarea.setBackground(Gatherer.config.getColor("coloring.editable_background", false));
	    // Connection
	    beta_checkbox.addActionListener(CollectionDesignManager.change_listener);
	    public_checkbox.addActionListener(CollectionDesignManager.change_listener);
	    creator_emailfield.getDocument().addDocumentListener(CollectionDesignManager.change_listener);
	    description_textarea.getDocument().addDocumentListener(CollectionDesignManager.change_listener);
	    icon_textfield.getDocument().addDocumentListener(CollectionDesignManager.change_listener);
	    maintainer_emailfield.getDocument().addDocumentListener(CollectionDesignManager.change_listener);
	    name_textfield.getDocument().addDocumentListener(CollectionDesignManager.change_listener);
	    small_icon_textfield.getDocument().addDocumentListener(CollectionDesignManager.change_listener);
	    Gatherer.dictionary.register(beta_checkbox, null, false);
	    Gatherer.dictionary.register(creator_label, null, false);
	    Gatherer.dictionary.register(description_label, null, false);
	    Gatherer.dictionary.register(icon_label, null, false);
	    Gatherer.dictionary.register(instructions_textarea, null, false);
	    Gatherer.dictionary.register(maintainer_label, null, false);
	    Gatherer.dictionary.register(name_label, null, false);
	    Gatherer.dictionary.register(public_checkbox, null, false);
	    Gatherer.dictionary.register(small_icon_label, null, false);
	    Gatherer.dictionary.register(title_label, null, false);
	    // Layout
	    instruction_panel.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
	    instruction_panel.setLayout(new BorderLayout());
	    instruction_panel.add(title_label, BorderLayout.NORTH);
	    instruction_panel.add(new JScrollPane(instructions_textarea), BorderLayout.CENTER);
	    
	    creator_panel.setLayout(new BorderLayout());
	    creator_panel.add(creator_label, BorderLayout.WEST);
	    creator_panel.add(creator_emailfield, BorderLayout.CENTER);

	    maintainer_panel.setLayout(new BorderLayout());
	    maintainer_panel.add(maintainer_label, BorderLayout.WEST);
	    maintainer_panel.add(maintainer_emailfield, BorderLayout.CENTER);

	    name_panel.setLayout(new BorderLayout());
	    name_panel.add(name_label, BorderLayout.WEST);
	    name_panel.add(name_textfield, BorderLayout.CENTER);

	    icon_panel.setLayout(new BorderLayout());
	    icon_panel.add(icon_label, BorderLayout.WEST);
	    icon_panel.add(icon_textfield, BorderLayout.CENTER);

	    small_icon_panel.setLayout(new BorderLayout());
	    small_icon_panel.add(small_icon_label, BorderLayout.WEST);
	    small_icon_panel.add(small_icon_textfield, BorderLayout.CENTER);

	    details_panel.setLayout(new GridLayout(7,1,5,0));
	    details_panel.add(creator_panel);
	    details_panel.add(maintainer_panel);
	    details_panel.add(public_checkbox);
	    details_panel.add(beta_checkbox);
	    details_panel.add(name_panel);
	    details_panel.add(icon_panel);
	    details_panel.add(small_icon_panel);

	    description_panel.setLayout(new BorderLayout());
	    description_panel.add(description_label, BorderLayout.NORTH);
	    description_panel.add(new JScrollPane(description_textarea), BorderLayout.CENTER);

	    all_details_panel.setLayout(new BorderLayout());
	    all_details_panel.add(details_panel, BorderLayout.NORTH);
	    all_details_panel.add(description_panel, BorderLayout.CENTER);

	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new BorderLayout());
	    add(instruction_panel, BorderLayout.NORTH);
	    add(all_details_panel, BorderLayout.CENTER);
	}
	/** Destructor. */
	public void destroy() {
	    Gatherer.dictionary.deregister(beta_checkbox);
	    Gatherer.dictionary.deregister(creator_label);
	    Gatherer.dictionary.deregister(description_label);
	    Gatherer.dictionary.deregister(icon_label);
	    Gatherer.dictionary.deregister(instructions_textarea);
	    Gatherer.dictionary.deregister(maintainer_label);
	    Gatherer.dictionary.deregister(name_label);
	    Gatherer.dictionary.deregister(public_checkbox);
	    Gatherer.dictionary.deregister(small_icon_label);
	    Gatherer.dictionary.deregister(title_label);
	}
	/** Called to refresh the components. */
	public void gainFocus() {
	    // Retrieve all of the elements that are dependant on default language.
	    collection_extra_collectionmeta = CollectionDesignManager.collectionmeta_manager.getMetadatum(CollectionConfiguration.COLLECTIONMETADATA_COLLECTIONEXTRA_STR);
	    collection_name_collectionmeta = CollectionDesignManager.collectionmeta_manager.getMetadatum(CollectionConfiguration.COLLECTIONMETADATA_COLLECTIONNAME_STR);
	    creator_collectionmeta = new CollectionMeta(CollectionDesignManager.collect_config.getCreator());
	    icon_collection_collectionmeta = CollectionDesignManager.collectionmeta_manager.getMetadatum(CollectionConfiguration.COLLECTIONMETADATA_ICONCOLLECTION_STR);
	    icon_collection_small_collectionmeta = CollectionDesignManager.collectionmeta_manager.getMetadatum(CollectionConfiguration.COLLECTIONMETADATA_ICONCOLLECTIONSMALL_STR);
	    maintainer_collectionmeta = new CollectionMeta(CollectionDesignManager.collect_config.getMaintainer());
	    // Make sure the components are up to date
	    creator_emailfield.setText(creator_collectionmeta.getValue(CollectionMeta.TEXT));
	    creator_emailfield.setCaretPosition(0);
	    description_textarea.setText(collection_extra_collectionmeta.getValue(CollectionMeta.TEXT));
	    description_textarea.setCaretPosition(0);
	    icon_textfield.setText(icon_collection_collectionmeta.getValue(CollectionMeta.TEXT));
	    icon_textfield.setCaretPosition(0);
	    maintainer_emailfield.setText(maintainer_collectionmeta.getValue(CollectionMeta.TEXT));
	    maintainer_emailfield.setCaretPosition(0);
	    name_textfield.setText(collection_name_collectionmeta.getValue(CollectionMeta.TEXT));
	    name_textfield.setCaretPosition(0);
	    small_icon_textfield.setText(icon_collection_small_collectionmeta.getValue(CollectionMeta.TEXT));
	    small_icon_textfield.setCaretPosition(0);
	    ready = true;
	}
	/** Called to store the current value of the components. */
	public void loseFocus() {
	    // String values. Have to test if this component has actually ever recieved focus anyway.
	    if(ready) {
		// Boolean values
		beta_collectionmeta.setValue((beta_checkbox.isSelected() ? CollectionConfiguration.TRUE_STR : CollectionConfiguration.FALSE_STR));
		public_collectionmeta.setValue((public_checkbox.isSelected() ? CollectionConfiguration.TRUE_STR : CollectionConfiguration.FALSE_STR));
		creator_collectionmeta.setValue(creator_emailfield.getText());
		collection_extra_collectionmeta.setValue(description_textarea.getText());
		icon_collection_collectionmeta.setValue(icon_textfield.getText());
		maintainer_collectionmeta.setValue(maintainer_emailfield.getText());
		collection_name_collectionmeta.setValue(name_textfield.getText());
		icon_collection_small_collectionmeta.setValue(small_icon_textfield.getText());
		ready = false;
	    }
	}

    }
    /** This tree provides a 'table of contents' for the various components of the design process (collection configuration in more technical terms). */
    private class DesignTree
	extends JTree {
	private DesignNode root = null;
	/** Constructor. Automatically generates all of the nodes, in the order of CONTENTS. */
	public DesignTree() {
	    super();
	    root = new DesignNode("Root");
	    this.setModel(new DefaultTreeModel(root));
	    // Now add the design categories.
	    for(int i = 0; i < CONTENTS.length; i++) {
		root.add(new DesignNode(CONTENTS[i]));
	    }
	    expandRow(0);
	    setRootVisible(false);
	    setSelectionRow(0);
	}
	/** Set the current view to the one specified.
	 * @param type the name of the desired view as a String
	 */
	public void setSelectedView(String type) {
	    type = Gatherer.dictionary.get(type);
	    for(int i = 0; i < root.getChildCount(); i++) {
		DesignNode child = (DesignNode) root.getChildAt(i);
		if(child.toString().equals(type)) {
		    TreePath path = new TreePath(child.getPath());
		    setSelectionPath(path);
		}
	    }
	}
    }
    /** A tree node that retains a reference to one of the possible design sub-views relating to the different sub-managers. */
    private class DesignNode
	extends DefaultMutableTreeNode {
	/** Constructor.
	 * @param object The <strong>Object</strong> assigned to this node.
		  */
	public DesignNode(String object) {
	    super(object);
	}
	/** Retrieve a textual representation of the object.
	 * @return a String
	 */
	public String toString() {
	    return get((String)getUserObject());
	}
    }
    /** Listens for selection changes in the 'contents' tree, and switches to the appropriate view. */
    private class TreeListener
	implements TreeSelectionListener {
	/** Called whenever the selection changes, we must update the view so it matches the node selected.
	 * @param event A <strong>TreeSelectionEvent</strong> containing more information about the tree selection.
	 * @see org.greenstone.gatherer.cdm.ClassifierManager
	 * @see org.greenstone.gatherer.cdm.CollectionDesignManager
	 * @see org.greenstone.gatherer.cdm.CollectionMetaManager
	 * @see org.greenstone.gatherer.cdm.FormatManager
	 * @see org.greenstone.gatherer.cdm.LanguageManager
	 * @see org.greenstone.gatherer.cdm.MetadataSetManager
	 * @see org.greenstone.gatherer.cdm.SubcollectionManager
	 * @see org.greenstone.gatherer.cdm.PlugInManager
	 */
	public void valueChanged(TreeSelectionEvent event) {
	    if(!tree.isSelectionEmpty()) {
		TreePath path = tree.getSelectionPath();
		DesignNode node = (DesignNode)path.getLastPathComponent();
		String type = (String)node.getUserObject();
		// Wait begins
		Gatherer.g_man.wait(true);
		// Save information in current view
		view.loseFocus();
		remove((JPanel)view);
		// Change panes.
		if(type == CONTENTS[0]) {
		    view = getControls();
		}
		else if(type == CONTENTS[1]) {
		    view = CollectionDesignManager.plugin_manager.getControls();
		}
		else if(type == CONTENTS[2]) {
		    view = CollectionDesignManager.searchtype_manager.getControls();
		}
		else if(type == CONTENTS[3]) {
		    view = CollectionDesignManager.index_manager.getControls();
		}
		else if(type == CONTENTS[4]) {
		    view = CollectionDesignManager.subcollection_manager.getControls();
		}
		else if(type == CONTENTS[5]) {
		    view = CollectionDesignManager.supercollection_manager.getControls();
		}
		else if(type == CONTENTS[6]) {
		    view = CollectionDesignManager.classifier_manager.getControls();
		}
		else if(type == CONTENTS[7]) {
		    view = CollectionDesignManager.format_manager.getControls();
		}
		else if(type == CONTENTS[8]) {
		    view = CollectionDesignManager.translation_view.getControls();
		}
		else if(type == CONTENTS[9]) {
		    view = CollectionDesignManager.metadataset_view.getControls();
		}
		add((JPanel)view, BorderLayout.CENTER);
		// Update information on visible pane
		view.gainFocus();
		repaint();
		// Wait ends
		Gatherer.g_man.wait(false);
	    }
	}
    }
}
