package org.greenstone.gatherer.shell;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.awt.Component;
import java.util.StringTokenizer;
import javax.swing.JProgressBar;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.shell.GShellProgressMonitor;
/** This implementation of <i>GShellProgressMonitor</i> is designed to parse and translate the progress of a import.pl call.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.1
 */
public class GImportProgressMonitor
    implements GShellProgressMonitor {
    /** Indicates if the GUI has asked the process this object monitors to stop. */
    private boolean stop = false;
    private int file_num = 0;
    /** The number of files expected to be scanned by this import process. */
    private int num_files = 0;
    /** This holds the number of documents actually processed by the import command, as garnered from the final block of text output. */
    private int num_docs = 0;
    /** The progress bar this monitor updates. */
    private JProgressBar progress_bar;
    /** The maximum value for this progress bar. */
    static final private int MAX = 1000000;
    /** The minimum value for this progress bar. */
    static final private int MIN = 0;
    /** A String fragment which is used to determine if the current output process line is referring to the number of documents actually processed by the import script. */
    static final private String NUM_DOCS_PROCESSED = "processed and included in the collection";
    /** The Sentinel value is the String fragment an output process line must start with to be considered a milestone. In this case we are counting directories, so the fragment refers to a directory scan. */
    static final private String SENTINEL = "recplug - ";

    public GImportProgressMonitor() {
	progress_bar = new JProgressBar();
	progress_bar.setIndeterminate(false);
	progress_bar.setMaximum(MAX);
	progress_bar.setMinimum(MIN);
	progress_bar.setStringPainted(true);
	progress_bar.setValue(MIN);
    }

    /** Method to register a new progress bar with this monitor.
     * @param progress_bar The new <strong>JProgressBar</strong>.
     */
    public void addProgressBar(JProgressBar progress_bar) {
	this.progress_bar = progress_bar;
	progress_bar.setMaximum(MAX);
	progress_bar.setMinimum(MIN);
	progress_bar.setValue(MIN);
    }

    /** Determine the script exit value according to the progress monitor. This gets around a problem where several script failures actually result with a successful exit value.
     * @return A <i>int</i> with a value of zero if and only if the script was successful.
     */
    public int exitValue() {
	if(num_docs > 0) {
	    return 0;
	}
	return 1;
    }

    /** Retrieve the number of documents recorded by the import monitor during the execution of the import scripts.
     * @return An <i>int</i> giving the document number.
     */
    public int getNumberOfDocuments() {
	return num_docs;
    }

    /** Method to retrieve whatever control is being used as the progress indicator. Usually a <strong>JProgressBar</strong> but there may be others implemented later.
     * @return A <strong>Component</strong> on which the progress of the process is being displayed.
     */
    public Component getProgress() {
	return progress_bar;
    }

    /** Method to determine the state of the stop flag, which may be set by the visual component that created this monitor.
     * @return A <strong>boolean</strong> indicating if the process has been asked to stop.
     */
    public boolean hasSignalledStop() {
	return stop;
    }

    /** Inform the progress bar that it should programatically increment progress by one step. This is only called during the metadata archive extraction so each step should be (1000000 / 5) / num_docs. */
    public void increment() {
	if(num_docs > 0) {
	    progress_bar.setValue(progress_bar.getValue() + ((MAX / 5) / num_docs));
	}
	// else {
	// progress_bar.setIndeterminate(true);
	// }
    }

    /** This method is used to 'feed in' a line of text captured from the process.
     * @param line A <strong>String</strong> of text captured from either standard out or standard error.
     * TODO Everthing.
     */
    public void parse(String line_raw) {
	progress_bar.setString("");
	String line = line_raw.toLowerCase();
	if (line.indexOf(SENTINEL) == 0) {
	    file_num = file_num + 1;
	    ///ystem.err.println("Done " + file_num + " of " + num_files + "...");
	    if (num_files > 0) {
		int base_val = MAX / 10;
		progress_bar.setValue(((((7 * MAX) / 10) * file_num) / num_files) + base_val);
	    }
	}
	else if (line.indexOf(NUM_DOCS_PROCESSED) != -1) {
	    num_docs = -1;
	    StringTokenizer tokenizer = new StringTokenizer(line);
	    while(num_docs == -1 && tokenizer.hasMoreTokens()) {
		String pos_num_docs_str = tokenizer.nextToken();
		try {
		    num_docs = Integer.parseInt(pos_num_docs_str);
		}
		catch (Exception error) {
		}
	    }
	    ///ystem.err.println("Parsed num_docs to be " + num_docs);
	}
    }

    public void saving() {
	progress_bar.setString(Gatherer.dictionary.get("SaveProgressDialog.Title", (String)null));
	progress_bar.setValue(MAX / (10 * 2));
    }

    /** Since the creator of this process monitor is actually in the GUI, this class provides the simpliest way to send a cancel process message between the two.
     * @param state The desired state of the stop flag as a <strong>boolean</strong>.
     */
    public void setStop(boolean state) {
	stop = state;
    }

    /** This method resets this monitor to the start, reseting the process parsing and progress bar.
     * TODO Everthing.
     */
    public void start() {
	progress_bar.setString(Gatherer.dictionary.get("FileActions.Calculating_Size", (String)null));
	progress_bar.setValue(MAX / 10);
	num_files = Gatherer.c_man.getCollection().getDocumentCount();
	///ystem.err.println("Number of files: " + num_files);
	file_num = 0;
    }

    /** This method indicates the process is complete. 
     * TODO Everthing.
     */
    public void stop() {
	progress_bar.setValue(MAX);
    }
}
